package iotdataplane_test

import (
	"errors"
	"testing"

	"github.com/aws/aws-sdk-go-v2-0.24/aws"
	"github.com/aws/aws-sdk-go-v2-0.24/aws/defaults"
	"github.com/aws/aws-sdk-go-v2-0.24/internal/awstesting/unit"
	"github.com/aws/aws-sdk-go-v2-0.24/service/iotdataplane"
)

func TestRequireEndpointIfRegionProvided(t *testing.T) {
	cfg := unit.Config()
	cfg.Handlers.Validate.Remove(defaults.ValidateParametersHandler)
	cfg.EndpointResolver = aws.ResolveWithEndpointURL("")

	svc := iotdataplane.New(cfg)
	req := svc.GetThingShadowRequest(nil)
	err := req.Build()

	if e, a := "", req.Endpoint.URL; e != a {
		t.Errorf("expect %v, got %v", e, a)
	}
	if err == nil {
		t.Errorf("expect error, got none")
	}
	var expected *aws.MissingEndpointError
	if !errors.As(err, &expected) {
		t.Fatalf("expected %T, got %T", expected, err)
	}
}

func TestRequireEndpointIfNoRegionProvided(t *testing.T) {
	cfg := unit.Config()
	cfg.Handlers.Validate.Remove(defaults.ValidateParametersHandler)
	cfg.EndpointResolver = aws.ResolveWithEndpointURL("")

	svc := iotdataplane.New(cfg)

	req := svc.GetThingShadowRequest(nil)
	err := req.Build()

	if e, a := "", req.Endpoint.URL; e != a {
		t.Errorf("expect %v, got %v", e, a)
	}
	if err == nil {
		t.Errorf("expect error, got none")
	}
	var expected *aws.MissingEndpointError
	if !errors.As(err, &expected) {
		t.Fatalf("expected %T, got %T", expected, err)
	}
}

func TestRequireEndpointUsed(t *testing.T) {
	cfg := unit.Config()
	cfg.Handlers.Validate.Remove(defaults.ValidateParametersHandler)
	cfg.EndpointResolver = aws.ResolveWithEndpointURL("https://endpoint")

	svc := iotdataplane.New(cfg)
	req := svc.GetThingShadowRequest(nil)
	err := req.Build()

	if e, a := "https://endpoint", req.Endpoint.URL; e != a {
		t.Errorf("expect %v, got %v", e, a)
	}
	if err != nil {
		t.Errorf("expect no error, got %v", err)
	}
}
